from sqlalchemy import Column, Integer, String, SmallInteger, DateTime, BigInteger
from sqlalchemy.ext.declarative import declarative_base
from datetime import datetime
Base = declarative_base()


class u_hero(Base):
    __tablename__ = 'u_hero'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, default=0)
    classe = Column("class", SmallInteger, default=0)
    name = Column(String, default="")
    hero_type = Column(SmallInteger, default=0)
    now_zone_idx = Column(SmallInteger, default=0)
    now_zone_x = Column(SmallInteger, default=0)
    now_zone_y = Column(SmallInteger, default=0)
    init_pos_layer = Column(SmallInteger, default=0)
    revive_zone_idx = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=1)
    gold = Column(Integer, default=0)
    attr = Column(Integer, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    abil_freepoint = Column(SmallInteger, default=0)
    res_fire = Column(SmallInteger, default=0)
    res_water = Column(SmallInteger, default=0)
    res_earth = Column(SmallInteger, default=0)
    res_wind = Column(SmallInteger, default=0)
    res_devil = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, default=0)
    regdate = Column(DateTime)
    avatar_head = Column(SmallInteger, default=0)
    avatar_body = Column(SmallInteger, default=0)
    avatar_foot = Column(SmallInteger, default=0)
    return_time = Column(Integer, default=0)
    status = Column(SmallInteger, default=0)
    status_time = Column(DateTime)
    nickname = Column(SmallInteger, default=0)
    last_logout_time = Column(DateTime)
    skill_point = Column(SmallInteger, default=0)
    login = Column(SmallInteger, default=0)
    limit_zone_idx = Column(SmallInteger, default=0)
    requestname = Column(SmallInteger, default=0)

    @property
    def formatted_regdate(self, regdate):
        return self.regdate.strftime("%Y-%m-%d %H:%M:%S")

    @property
    def formatted_status_time(self, status_time):
        return self.status_time.strftime("%Y-%m-%d %H:%M:%S")

    @property
    def formatted_last_logout_time(self, last_logout_time):
        return self.last_logout_time.strftime("%Y-%m-%d %H:%M:%S")


class u_CastleWarInfo(Base):
    __tablename__ = 'u_CastleWarInfo'
    zone_idx = Column(Integer, primary_key=True)
    entry_type = Column(SmallInteger, primary_key=True)
    guild_idx = Column(BigInteger, primary_key=True)
    guild_name = Column(String)


class u_guild(Base):
    __tablename__ = 'u_guild'
    GuildIdx = Column(Integer, primary_key=True)
    Name = Column(String)
    Info = Column(String)
    Cert = Column(String)
    EstablishDate = Column(DateTime)
    LimitCount = Column(SmallInteger)
    Status = Column(SmallInteger)
    MarkRegDate = Column(DateTime)
    MarkRegCnt = Column(SmallInteger)
    Dissolution = Column(DateTime)
    gold = Column(Integer)
    HiringIdx = Column(SmallInteger)
    CertDate = Column(DateTime)
    InfoDate = Column(DateTime)

    @property
    def formatted_EstablishDate(self, EstablishDate):
        return self.EstablishDate.strftime("%Y-%m-%d %H:%M:%S")

    @property
    def formatted_MarkRegDate(self, MarkRegDate):
        return self.MarkRegDate.strftime("%Y-%m-%d %H:%M:%S")

    @property
    def formatted_Dissolution(self, Dissolution):
        return self.Dissolution.strftime("%Y-%m-%d %H:%M:%S")

    @property
    def formatted_CertDate(self, CertDate):
        return self.CertDate.strftime("%Y-%m-%d %H:%M:%S")

    @property
    def formatted_InfoDate(self, InfoDate):
        return self.InfoDate.strftime("%Y-%m-%d %H:%M:%S")


class u_guildmember(Base):
    __tablename__ = 'u_guildmember'
    HeroIdx = Column(Integer, primary_key=True)
    HeroOrder = Column(SmallInteger, primary_key=True)
    GuildIdx = Column(SmallInteger, default=0)
    MemberID = Column(SmallInteger, default=0)
    Grade = Column(SmallInteger, nullable=False)
    Authority = Column(Integer, nullable=False)
    UpgradeDate = Column(String, default="0")
    Memo = Column(String, default="")


class u_GuildZone(Base):
    __tablename__ = 'u_GuildZone'
    zone_idx = Column(SmallInteger, primary_key=True)
    guild_idx = Column(SmallInteger, default=0)
    tax_rate = Column(SmallInteger, default=100)
    tax_change_count = Column(SmallInteger, default=100)
    tax_check_time = Column(DateTime, nullable=False)

    @property
    def formatted_tax_check_time(self, tax_check_time):
        return self.tax_check_time.strftime("%Y-%m-%d %H:%M:%S")


class u_hench_0(Base):
    __tablename__ = 'u_hench_0'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_1(Base):
    __tablename__ = 'u_hench_1'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_2(Base):
    __tablename__ = 'u_hench_2'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_3(Base):
    __tablename__ = 'u_hench_3'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_4(Base):
    __tablename__ = 'u_hench_4'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_5(Base):
    __tablename__ = 'u_hench_5'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_6(Base):
    __tablename__ = 'u_hench_6'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_7(Base):
    __tablename__ = 'u_hench_7'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_8(Base):
    __tablename__ = 'u_hench_8'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hench_9(Base):
    __tablename__ = 'u_hench_9'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    position = Column(SmallInteger, default=0)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_skill = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_0(Base):
    __tablename__ = 'u_store_hench_0'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_1(Base):
    __tablename__ = 'u_store_hench_1'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_2(Base):
    __tablename__ = 'u_store_hench_2'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_3(Base):
    __tablename__ = 'u_store_hench_3'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_4(Base):
    __tablename__ = 'u_store_hench_4'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_5(Base):
    __tablename__ = 'u_store_hench_5'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_6(Base):
    __tablename__ = 'u_store_hench_6'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_7(Base):
    __tablename__ = 'u_store_hench_7'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_8(Base):
    __tablename__ = 'u_store_hench_8'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_store_hench_9(Base):
    __tablename__ = 'u_store_hench_9'
    id_idx = Column(Integer, primary_key=True)
    serial = Column(BigInteger, primary_key=True)
    hench_order = Column(SmallInteger, default=0)
    monster_type = Column(SmallInteger, default=0)
    name = Column(String, default="")
    sex = Column(SmallInteger, default=0)
    state = Column(SmallInteger, default=0)
    mixnum = Column(SmallInteger, default=0)
    baselevel = Column(SmallInteger, default=0)
    max_baselevel = Column(SmallInteger, default=0)
    exp = Column(BigInteger, default=0)
    speed_move = Column(SmallInteger, default=0)
    speed_attack = Column(SmallInteger, default=0)
    speed_spell = Column(SmallInteger, default=0)
    str = Column(SmallInteger, default=0)
    dex = Column(SmallInteger, default=0)
    aim = Column(SmallInteger, default=0)
    luck = Column(SmallInteger, default=0)
    ap = Column(SmallInteger, default=0)
    dp = Column(SmallInteger, default=0)
    hc = Column(SmallInteger, default=0)
    hd = Column(SmallInteger, default=0)
    hp = Column(SmallInteger, default=0)
    mp = Column(SmallInteger, default=0)
    maxhp = Column(SmallInteger, default=0)
    maxmp = Column(SmallInteger, default=0)
    item0 = Column(Integer, default=0)
    item1 = Column(Integer, default=0)
    item2 = Column(Integer, default=0)
    growthtype = Column(SmallInteger, default=0)
    race_val = Column(SmallInteger, default=0)
    ign_att_cnt = Column(SmallInteger, nullable=False)
    add_defense_cnt = Column(SmallInteger, default=0)
    enchant_grade = Column(SmallInteger, nullable=False)
    item_slot_total = Column(SmallInteger, default=0)
    item_idx_0 = Column(SmallInteger, default=0)
    item_serial_0 = Column(BigInteger, default=0)
    item_duration_0 = Column(Integer, default=0)
    item_idx_1 = Column(SmallInteger, default=0)
    item_serial_1 = Column(BigInteger, default=0)
    item_duration_1 = Column(Integer, default=0)
    item_idx_2 = Column(SmallInteger, default=0)
    item_serial_2 = Column(BigInteger, default=0)
    item_duration_2 = Column(Integer, default=0)
    duration = Column(Integer, nullable=False)
    last_check_time = Column(DateTime, default='0000-00-00 00:00:00')


class u_hero_quest(Base):
    __tablename__ = 'u_hero_quest'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    switch_number = Column(SmallInteger, primary_key=True, default=0)
    success_count = Column(SmallInteger, nullable=False)
    success_time = Column(
        DateTime, default=datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

    @property
    def formatted_success_time(self, success_time):
        return self.success_time.strftime("%Y-%m-%d %H:%M:%S")


class u_HeroSkill(Base):
    __tablename__ = 'u_HeroSkill'
    heroIndex = Column(Integer, primary_key=True)
    heroSocketNum = Column(SmallInteger, primary_key=True)
    skillIndex = Column(SmallInteger, default=0, primary_key=True)
    skillLevel = Column(SmallInteger, default=0)
    skillPoint = Column(SmallInteger, default=0)
    learningDate = Column(DateTime, default='0000-00-00 00:00:00')

    @property
    def formatted_learningDate(self, learningDate):
        return self.learningDate.strftime("%Y-%m-%d %H:%M:%S")


class u_item(Base):
    __tablename__ = 'u_item'
    id_idx = Column(String, primary_key=True)
    hero_order = Column(String, primary_key=True)
    serial = Column(String, primary_key=True)
    item_idx = Column(String, nullable=False)
    socket_type = Column(String, nullable=False)
    socket_num = Column(String, primary_key=True)
    count = Column(String, nullable=False)
    opt = Column(String, nullable=False)
    opt_level = Column(String, nullable=False)
    duration = Column(String, nullable=False)
    last_check_time = Column(String, nullable=False)
    synergy = Column(String, nullable=False)
    synergy_level = Column(String, nullable=False)


class u_store_item(Base):
    __tablename__ = 'u_store_item'
    id_idx = Column(String, primary_key=True)
    serial = Column(String, primary_key=True)
    item_idx = Column(String, nullable=False)
    socket_num = Column(String, primary_key=True)
    count = Column(String, nullable=False)
    opt = Column(String, nullable=False)
    opt_level = Column(String, nullable=False)
    duration = Column(String, nullable=False)
    last_check_time = Column(String, nullable=False)
    synergy = Column(String, nullable=False)
    synergy_level = Column(String, nullable=False)


class u_messenger(Base):
    __tablename__ = 'u_messenger'
    HeroIdx = Column(Integer, primary_key=True)
    HeroOrder = Column(SmallInteger, primary_key=True)
    TargetHeroIdx = Column(Integer, primary_key=True)
    TargetHeroOrder = Column(SmallInteger, primary_key=True)
    Status = Column(SmallInteger, default=0)
    MyState = Column(SmallInteger, default=1)


class u_MixLog(Base):
    __tablename__ = 'u_MixLog'
    HeroIdx = Column(Integer, primary_key=True)
    HeroOrder = Column(SmallInteger, primary_key=True)
    type = Column(Integer, nullable=False)
    SuccessCount = Column(SmallInteger, nullable=False)
    FirstSuccessTime = Column(DateTime, nullable=False)

    @property
    def formatted_FirstSuccessTime(self, FirstSuccessTime):
        return self.FirstSuccessTime.strftime("%Y-%m-%d %H:%M:%S")


class u_MixSkill(Base):
    __tablename__ = 'u_MixSkill'
    HeroIdx = Column(Integer, primary_key=True)
    HeroOrder = Column(SmallInteger, primary_key=True)
    MixSkill1 = Column(SmallInteger, default=0)
    MixSkill2 = Column(SmallInteger, default=0)
    MixSkill3 = Column(SmallInteger, default=0)
    MixSkill4 = Column(SmallInteger, default=0)
    MixSkill5 = Column(SmallInteger, default=0)
    MixSkill6 = Column(SmallInteger, default=0)
    MixSkill7 = Column(SmallInteger, default=0)
    MixSkill8 = Column(SmallInteger, default=0)
    MixSkill9 = Column(SmallInteger, default=0)
    MixSkill10 = Column(SmallInteger, default=0)


class u_QuestLog(Base):
    __tablename__ = 'u_QuestLog'
    id_idx = Column(Integer, primary_key=True)
    hero_order = Column(SmallInteger, primary_key=True)
    quest_index = Column(SmallInteger, primary_key=True)
    quest_state = Column(SmallInteger, default=0)
    start_time = Column(DateTime, default='0000-00-00 00:00:00')
    end_time = Column(DateTime, default='0000-00-00 00:00:00')
    end_count = Column(SmallInteger, default=0)

    @property
    def formatted_start_time(self, start_time):
        return self.start_time.strftime("%Y-%m-%d %H:%M:%S")

    @property
    def formatted_end_time(self, end_time):
        return self.end_time.strftime("%Y-%m-%d %H:%M:%S")


class u_store(Base):
    __tablename__ = 'u_store'
    id_idx = Column(Integer, primary_key=True)
    store_gold = Column(Integer, default=0)
    store_state = Column(Integer, default=0)
    last_store_time = Column(DateTime, default='0000-00-00 00:00:00')

    @property
    def formatted_last_store_time(self, last_store_time):
        return self.last_store_time.strftime("%Y-%m-%d %H:%M:%S")
